PG_TYPES = [
    {"typname": "bool", "oid": 16},
    {"typname": "bytea", "oid": 17},
    {"typname": "char", "oid": 18},
    {"typname": "name", "oid": 19},
    {"typname": "int8", "oid": 20},
    {"typname": "int2", "oid": 21},
    {"typname": "int2vector", "oid": 22},
    {"typname": "int4", "oid": 23},
    {"typname": "regproc", "oid": 24},
    {"typname": "text", "oid": 25},
    {"typname": "oid", "oid": 26},
    {"typname": "tid", "oid": 27},
    {"typname": "xid", "oid": 28},
    {"typname": "cid", "oid": 29},
    {"typname": "oidvector", "oid": 30},
    {"typname": "pg_ddl_command", "oid": 32},
    {"typname": "pg_type", "oid": 71},
    {"typname": "pg_attribute", "oid": 75},
    {"typname": "pg_proc", "oid": 81},
    {"typname": "pg_class", "oid": 83},
    {"typname": "json", "oid": 114},
    {"typname": "xml", "oid": 142},
    {"typname": "_xml", "oid": 143},
    {"typname": "pg_node_tree", "oid": 194},
    {"typname": "_json", "oid": 199},
    {"typname": "_pg_type", "oid": 210},
    {"typname": "table_am_handler", "oid": 269},
    {"typname": "_pg_attribute", "oid": 270},
    {"typname": "_xid8", "oid": 271},
    {"typname": "_pg_proc", "oid": 272},
    {"typname": "_pg_class", "oid": 273},
    {"typname": "index_am_handler", "oid": 325},
    {"typname": "point", "oid": 600},
    {"typname": "lseg", "oid": 601},
    {"typname": "path", "oid": 602},
    {"typname": "box", "oid": 603},
    {"typname": "polygon", "oid": 604},
    {"typname": "line", "oid": 628},
    {"typname": "_line", "oid": 629},
    {"typname": "cidr", "oid": 650},
    {"typname": "_cidr", "oid": 651},
    {"typname": "float4", "oid": 700},
    {"typname": "float8", "oid": 701},
    {"typname": "unknown", "oid": 705},
    {"typname": "circle", "oid": 718},
    {"typname": "_circle", "oid": 719},
    {"typname": "macaddr8", "oid": 774},
    {"typname": "_macaddr8", "oid": 775},
    {"typname": "money", "oid": 790},
    {"typname": "_money", "oid": 791},
    {"typname": "macaddr", "oid": 829},
    {"typname": "inet", "oid": 869},
    {"typname": "_bool", "oid": 1000},
    {"typname": "_bytea", "oid": 1001},
    {"typname": "_char", "oid": 1002},
    {"typname": "_name", "oid": 1003},
    {"typname": "_int2", "oid": 1005},
    {"typname": "_int2vector", "oid": 1006},
    {"typname": "_int4", "oid": 1007},
    {"typname": "_regproc", "oid": 1008},
    {"typname": "_text", "oid": 1009},
    {"typname": "_tid", "oid": 1010},
    {"typname": "_xid", "oid": 1011},
    {"typname": "_cid", "oid": 1012},
    {"typname": "_oidvector", "oid": 1013},
    {"typname": "_bpchar", "oid": 1014},
    {"typname": "_varchar", "oid": 1015},
    {"typname": "_int8", "oid": 1016},
    {"typname": "_point", "oid": 1017},
    {"typname": "_lseg", "oid": 1018},
    {"typname": "_path", "oid": 1019},
    {"typname": "_box", "oid": 1020},
    {"typname": "_float4", "oid": 1021},
    {"typname": "_float8", "oid": 1022},
    {"typname": "_polygon", "oid": 1027},
    {"typname": "_oid", "oid": 1028},
    {"typname": "aclitem", "oid": 1033},
    {"typname": "_aclitem", "oid": 1034},
    {"typname": "_macaddr", "oid": 1040},
    {"typname": "_inet", "oid": 1041},
    {"typname": "bpchar", "oid": 1042},
    {"typname": "varchar", "oid": 1043},
    {"typname": "date", "oid": 1082},
    {"typname": "time", "oid": 1083},
    {"typname": "timestamp", "oid": 1114},
    {"typname": "_timestamp", "oid": 1115},
    {"typname": "_date", "oid": 1182},
    {"typname": "_time", "oid": 1183},
    {"typname": "timestamptz", "oid": 1184},
    {"typname": "_timestamptz", "oid": 1185},
    {"typname": "interval", "oid": 1186},
    {"typname": "_interval", "oid": 1187},
    {"typname": "_numeric", "oid": 1231},
    {"typname": "pg_database", "oid": 1248},
    {"typname": "_cstring", "oid": 1263},
    {"typname": "timetz", "oid": 1266},
    {"typname": "_timetz", "oid": 1270},
    {"typname": "bit", "oid": 1560},
    {"typname": "_bit", "oid": 1561},
    {"typname": "varbit", "oid": 1562},
    {"typname": "_varbit", "oid": 1563},
    {"typname": "numeric", "oid": 1700},
    {"typname": "refcursor", "oid": 1790},
    {"typname": "_refcursor", "oid": 2201},
    {"typname": "regprocedure", "oid": 2202},
    {"typname": "regoper", "oid": 2203},
    {"typname": "regoperator", "oid": 2204},
    {"typname": "regclass", "oid": 2205},
    {"typname": "regtype", "oid": 2206},
    {"typname": "_regprocedure", "oid": 2207},
    {"typname": "_regoper", "oid": 2208},
    {"typname": "_regoperator", "oid": 2209},
    {"typname": "_regclass", "oid": 2210},
    {"typname": "_regtype", "oid": 2211},
    {"typname": "record", "oid": 2249},
    {"typname": "cstring", "oid": 2275},
    {"typname": "any", "oid": 2276},
    {"typname": "anyarray", "oid": 2277},
    {"typname": "void", "oid": 2278},
    {"typname": "trigger", "oid": 2279},
    {"typname": "language_handler", "oid": 2280},
    {"typname": "internal", "oid": 2281},
    {"typname": "anyelement", "oid": 2283},
    {"typname": "_record", "oid": 2287},
    {"typname": "anynonarray", "oid": 2776},
    {"typname": "pg_authid", "oid": 2842},
    {"typname": "pg_auth_members", "oid": 2843},
    {"typname": "_txid_snapshot", "oid": 2949},
    {"typname": "uuid", "oid": 2950},
    {"typname": "_uuid", "oid": 2951},
    {"typname": "txid_snapshot", "oid": 2970},
    {"typname": "fdw_handler", "oid": 3115},
    {"typname": "pg_lsn", "oid": 3220},
    {"typname": "_pg_lsn", "oid": 3221},
    {"typname": "tsm_handler", "oid": 3310},
    {"typname": "pg_ndistinct", "oid": 3361},
    {"typname": "pg_dependencies", "oid": 3402},
    {"typname": "anyenum", "oid": 3500},
    {"typname": "tsvector", "oid": 3614},
    {"typname": "tsquery", "oid": 3615},
    {"typname": "gtsvector", "oid": 3642},
    {"typname": "_tsvector", "oid": 3643},
    {"typname": "_gtsvector", "oid": 3644},
    {"typname": "_tsquery", "oid": 3645},
    {"typname": "regconfig", "oid": 3734},
    {"typname": "_regconfig", "oid": 3735},
    {"typname": "regdictionary", "oid": 3769},
    {"typname": "_regdictionary", "oid": 3770},
    {"typname": "jsonb", "oid": 3802},
    {"typname": "_jsonb", "oid": 3807},
    {"typname": "anyrange", "oid": 3831},
    {"typname": "event_trigger", "oid": 3838},
    {"typname": "int4range", "oid": 3904},
    {"typname": "_int4range", "oid": 3905},
    {"typname": "numrange", "oid": 3906},
    {"typname": "_numrange", "oid": 3907},
    {"typname": "tsrange", "oid": 3908},
    {"typname": "_tsrange", "oid": 3909},
    {"typname": "tstzrange", "oid": 3910},
    {"typname": "_tstzrange", "oid": 3911},
    {"typname": "daterange", "oid": 3912},
    {"typname": "_daterange", "oid": 3913},
    {"typname": "int8range", "oid": 3926},
    {"typname": "_int8range", "oid": 3927},
    {"typname": "pg_shseclabel", "oid": 4066},
    {"typname": "jsonpath", "oid": 4072},
    {"typname": "_jsonpath", "oid": 4073},
    {"typname": "regnamespace", "oid": 4089},
    {"typname": "_regnamespace", "oid": 4090},
    {"typname": "regrole", "oid": 4096},
    {"typname": "_regrole", "oid": 4097},
    {"typname": "regcollation", "oid": 4191},
    {"typname": "_regcollation", "oid": 4192},
    {"typname": "int4multirange", "oid": 4451},
    {"typname": "nummultirange", "oid": 4532},
    {"typname": "tsmultirange", "oid": 4533},
    {"typname": "tstzmultirange", "oid": 4534},
    {"typname": "datemultirange", "oid": 4535},
    {"typname": "int8multirange", "oid": 4536},
    {"typname": "anymultirange", "oid": 4537},
    {"typname": "anycompatiblemultirange", "oid": 4538},
    {"typname": "pg_brin_bloom_summary", "oid": 4600},
    {"typname": "pg_brin_minmax_multi_summary", "oid": 4601},
    {"typname": "pg_mcv_list", "oid": 5017},
    {"typname": "pg_snapshot", "oid": 5038},
    {"typname": "_pg_snapshot", "oid": 5039},
    {"typname": "xid8", "oid": 5069},
    {"typname": "anycompatible", "oid": 5077},
    {"typname": "anycompatiblearray", "oid": 5078},
    {"typname": "anycompatiblenonarray", "oid": 5079},
    {"typname": "anycompatiblerange", "oid": 5080},
    {"typname": "pg_subscription", "oid": 6101},
    {"typname": "_int4multirange", "oid": 6150},
    {"typname": "_nummultirange", "oid": 6151},
    {"typname": "_tsmultirange", "oid": 6152},
    {"typname": "_tstzmultirange", "oid": 6153},
    {"typname": "_datemultirange", "oid": 6155},
    {"typname": "_int8multirange", "oid": 6157},
    {"typname": "_pg_attrdef", "oid": 10000},
    {"typname": "pg_attrdef", "oid": 10001},
    {"typname": "_pg_constraint", "oid": 10002},
    {"typname": "pg_constraint", "oid": 10003},
    {"typname": "_pg_inherits", "oid": 10004},
    {"typname": "pg_inherits", "oid": 10005},
    {"typname": "_pg_index", "oid": 10006},
    {"typname": "pg_index", "oid": 10007},
    {"typname": "_pg_operator", "oid": 10008},
    {"typname": "pg_operator", "oid": 10009},
    {"typname": "_pg_opfamily", "oid": 10010},
    {"typname": "pg_opfamily", "oid": 10011},
    {"typname": "_pg_opclass", "oid": 10012},
    {"typname": "pg_opclass", "oid": 10013},
    {"typname": "_pg_am", "oid": 10014},
    {"typname": "pg_am", "oid": 10015},
    {"typname": "_pg_amop", "oid": 10016},
    {"typname": "pg_amop", "oid": 10017},
    {"typname": "_pg_amproc", "oid": 10018},
    {"typname": "pg_amproc", "oid": 10019},
    {"typname": "_pg_language", "oid": 10020},
    {"typname": "pg_language", "oid": 10021},
    {"typname": "_pg_largeobject_metadata", "oid": 10022},
    {"typname": "pg_largeobject_metadata", "oid": 10023},
    {"typname": "_pg_largeobject", "oid": 10024},
    {"typname": "pg_largeobject", "oid": 10025},
    {"typname": "_pg_aggregate", "oid": 10026},
    {"typname": "pg_aggregate", "oid": 10027},
    {"typname": "_pg_statistic", "oid": 10028},
    {"typname": "pg_statistic", "oid": 10029},
    {"typname": "_pg_statistic_ext", "oid": 10030},
    {"typname": "pg_statistic_ext", "oid": 10031},
    {"typname": "_pg_statistic_ext_data", "oid": 10032},
    {"typname": "pg_statistic_ext_data", "oid": 10033},
    {"typname": "_pg_rewrite", "oid": 10034},
    {"typname": "pg_rewrite", "oid": 10035},
    {"typname": "_pg_trigger", "oid": 10036},
    {"typname": "pg_trigger", "oid": 10037},
    {"typname": "_pg_event_trigger", "oid": 10038},
    {"typname": "pg_event_trigger", "oid": 10039},
    {"typname": "_pg_description", "oid": 10040},
    {"typname": "pg_description", "oid": 10041},
    {"typname": "_pg_cast", "oid": 10042},
    {"typname": "pg_cast", "oid": 10043},
    {"typname": "_pg_enum", "oid": 10044},
    {"typname": "pg_enum", "oid": 10045},
    {"typname": "_pg_namespace", "oid": 10046},
    {"typname": "pg_namespace", "oid": 10047},
    {"typname": "_pg_conversion", "oid": 10048},
    {"typname": "pg_conversion", "oid": 10049},
    {"typname": "_pg_depend", "oid": 10050},
    {"typname": "pg_depend", "oid": 10051},
    {"typname": "_pg_database", "oid": 10052},
    {"typname": "_pg_db_role_setting", "oid": 10053},
    {"typname": "pg_db_role_setting", "oid": 10054},
    {"typname": "_pg_tablespace", "oid": 10055},
    {"typname": "pg_tablespace", "oid": 10056},
    {"typname": "_pg_authid", "oid": 10057},
    {"typname": "_pg_auth_members", "oid": 10058},
    {"typname": "_pg_shdepend", "oid": 10059},
    {"typname": "pg_shdepend", "oid": 10060},
    {"typname": "_pg_shdescription", "oid": 10061},
    {"typname": "pg_shdescription", "oid": 10062},
    {"typname": "_pg_ts_config", "oid": 10063},
    {"typname": "pg_ts_config", "oid": 10064},
    {"typname": "_pg_ts_config_map", "oid": 10065},
    {"typname": "pg_ts_config_map", "oid": 10066},
    {"typname": "_pg_ts_dict", "oid": 10067},
    {"typname": "pg_ts_dict", "oid": 10068},
    {"typname": "_pg_ts_parser", "oid": 10069},
    {"typname": "pg_ts_parser", "oid": 10070},
    {"typname": "_pg_ts_template", "oid": 10071},
    {"typname": "pg_ts_template", "oid": 10072},
    {"typname": "_pg_extension", "oid": 10073},
    {"typname": "pg_extension", "oid": 10074},
    {"typname": "_pg_foreign_data_wrapper", "oid": 10075},
    {"typname": "pg_foreign_data_wrapper", "oid": 10076},
    {"typname": "_pg_foreign_server", "oid": 10077},
    {"typname": "pg_foreign_server", "oid": 10078},
    {"typname": "_pg_user_mapping", "oid": 10079},
    {"typname": "pg_user_mapping", "oid": 10080},
    {"typname": "_pg_foreign_table", "oid": 10081},
    {"typname": "pg_foreign_table", "oid": 10082},
    {"typname": "_pg_policy", "oid": 10083},
    {"typname": "pg_policy", "oid": 10084},
    {"typname": "_pg_replication_origin", "oid": 10085},
    {"typname": "pg_replication_origin", "oid": 10086},
    {"typname": "_pg_default_acl", "oid": 10087},
    {"typname": "pg_default_acl", "oid": 10088},
    {"typname": "_pg_init_privs", "oid": 10089},
    {"typname": "pg_init_privs", "oid": 10090},
    {"typname": "_pg_seclabel", "oid": 10091},
    {"typname": "pg_seclabel", "oid": 10092},
    {"typname": "_pg_shseclabel", "oid": 10093},
    {"typname": "_pg_collation", "oid": 10094},
    {"typname": "pg_collation", "oid": 10095},
    {"typname": "_pg_parameter_acl", "oid": 10096},
    {"typname": "pg_parameter_acl", "oid": 10097},
    {"typname": "_pg_partitioned_table", "oid": 10098},
    {"typname": "pg_partitioned_table", "oid": 10099},
    {"typname": "_pg_range", "oid": 10100},
    {"typname": "pg_range", "oid": 10101},
    {"typname": "_pg_transform", "oid": 10102},
    {"typname": "pg_transform", "oid": 10103},
    {"typname": "_pg_sequence", "oid": 10104},
    {"typname": "pg_sequence", "oid": 10105},
    {"typname": "_pg_publication", "oid": 10106},
    {"typname": "pg_publication", "oid": 10107},
    {"typname": "_pg_publication_namespace", "oid": 10108},
    {"typname": "pg_publication_namespace", "oid": 10109},
    {"typname": "_pg_publication_rel", "oid": 10110},
    {"typname": "pg_publication_rel", "oid": 10111},
    {"typname": "_pg_subscription", "oid": 10112},
    {"typname": "_pg_subscription_rel", "oid": 10113},
    {"typname": "pg_subscription_rel", "oid": 10114},
    {"typname": "_pg_roles", "oid": 12001},
    {"typname": "pg_roles", "oid": 12002},
    {"typname": "_pg_shadow", "oid": 12006},
    {"typname": "pg_shadow", "oid": 12007},
    {"typname": "_pg_group", "oid": 12011},
    {"typname": "pg_group", "oid": 12012},
    {"typname": "_pg_user", "oid": 12015},
    {"typname": "pg_user", "oid": 12016},
    {"typname": "_pg_policies", "oid": 12019},
    {"typname": "pg_policies", "oid": 12020},
    {"typname": "_pg_rules", "oid": 12024},
    {"typname": "pg_rules", "oid": 12025},
    {"typname": "_pg_views", "oid": 12029},
    {"typname": "pg_views", "oid": 12030},
    {"typname": "_pg_tables", "oid": 12034},
    {"typname": "pg_tables", "oid": 12035},
    {"typname": "_pg_matviews", "oid": 12039},
    {"typname": "pg_matviews", "oid": 12040},
    {"typname": "_pg_indexes", "oid": 12044},
    {"typname": "pg_indexes", "oid": 12045},
    {"typname": "_pg_sequences", "oid": 12049},
    {"typname": "pg_sequences", "oid": 12050},
    {"typname": "_pg_stats", "oid": 12054},
    {"typname": "pg_stats", "oid": 12055},
    {"typname": "_pg_stats_ext", "oid": 12059},
    {"typname": "pg_stats_ext", "oid": 12060},
    {"typname": "_pg_stats_ext_exprs", "oid": 12064},
    {"typname": "pg_stats_ext_exprs", "oid": 12065},
    {"typname": "_pg_publication_tables", "oid": 12069},
    {"typname": "pg_publication_tables", "oid": 12070},
    {"typname": "_pg_locks", "oid": 12074},
    {"typname": "pg_locks", "oid": 12075},
    {"typname": "_pg_cursors", "oid": 12078},
    {"typname": "pg_cursors", "oid": 12079},
    {"typname": "_pg_available_extensions", "oid": 12082},
    {"typname": "pg_available_extensions", "oid": 12083},
    {"typname": "_pg_available_extension_versions", "oid": 12086},
    {"typname": "pg_available_extension_versions", "oid": 12087},
    {"typname": "_pg_prepared_xacts", "oid": 12091},
    {"typname": "pg_prepared_xacts", "oid": 12092},
    {"typname": "_pg_prepared_statements", "oid": 12096},
    {"typname": "pg_prepared_statements", "oid": 12097},
    {"typname": "_pg_seclabels", "oid": 12100},
    {"typname": "pg_seclabels", "oid": 12101},
    {"typname": "_pg_settings", "oid": 12105},
    {"typname": "pg_settings", "oid": 12106},
    {"typname": "_pg_file_settings", "oid": 12111},
    {"typname": "pg_file_settings", "oid": 12112},
    {"typname": "_pg_hba_file_rules", "oid": 12115},
    {"typname": "pg_hba_file_rules", "oid": 12116},
    {"typname": "_pg_ident_file_mappings", "oid": 12119},
    {"typname": "pg_ident_file_mappings", "oid": 12120},
    {"typname": "_pg_timezone_abbrevs", "oid": 12123},
    {"typname": "pg_timezone_abbrevs", "oid": 12124},
    {"typname": "_pg_timezone_names", "oid": 12127},
    {"typname": "pg_timezone_names", "oid": 12128},
    {"typname": "_pg_config", "oid": 12131},
    {"typname": "pg_config", "oid": 12132},
    {"typname": "_pg_shmem_allocations", "oid": 12135},
    {"typname": "pg_shmem_allocations", "oid": 12136},
    {"typname": "_pg_backend_memory_contexts", "oid": 12139},
    {"typname": "pg_backend_memory_contexts", "oid": 12140},
    {"typname": "_pg_stat_all_tables", "oid": 12143},
    {"typname": "pg_stat_all_tables", "oid": 12144},
    {"typname": "_pg_stat_xact_all_tables", "oid": 12148},
    {"typname": "pg_stat_xact_all_tables", "oid": 12149},
    {"typname": "_pg_stat_sys_tables", "oid": 12153},
    {"typname": "pg_stat_sys_tables", "oid": 12154},
    {"typname": "_pg_stat_xact_sys_tables", "oid": 12158},
    {"typname": "pg_stat_xact_sys_tables", "oid": 12159},
    {"typname": "_pg_stat_user_tables", "oid": 12162},
    {"typname": "pg_stat_user_tables", "oid": 12163},
    {"typname": "_pg_stat_xact_user_tables", "oid": 12167},
    {"typname": "pg_stat_xact_user_tables", "oid": 12168},
    {"typname": "_pg_statio_all_tables", "oid": 12171},
    {"typname": "pg_statio_all_tables", "oid": 12172},
    {"typname": "_pg_statio_sys_tables", "oid": 12176},
    {"typname": "pg_statio_sys_tables", "oid": 12177},
    {"typname": "_pg_statio_user_tables", "oid": 12180},
    {"typname": "pg_statio_user_tables", "oid": 12181},
    {"typname": "_pg_stat_all_indexes", "oid": 12184},
    {"typname": "pg_stat_all_indexes", "oid": 12185},
    {"typname": "_pg_stat_sys_indexes", "oid": 12189},
    {"typname": "pg_stat_sys_indexes", "oid": 12190},
    {"typname": "_pg_stat_user_indexes", "oid": 12193},
    {"typname": "pg_stat_user_indexes", "oid": 12194},
    {"typname": "_pg_statio_all_indexes", "oid": 12197},
    {"typname": "pg_statio_all_indexes", "oid": 12198},
    {"typname": "_pg_statio_sys_indexes", "oid": 12202},
    {"typname": "pg_statio_sys_indexes", "oid": 12203},
    {"typname": "_pg_statio_user_indexes", "oid": 12206},
    {"typname": "pg_statio_user_indexes", "oid": 12207},
    {"typname": "_pg_statio_all_sequences", "oid": 12210},
    {"typname": "pg_statio_all_sequences", "oid": 12211},
    {"typname": "_pg_statio_sys_sequences", "oid": 12215},
    {"typname": "pg_statio_sys_sequences", "oid": 12216},
    {"typname": "_pg_statio_user_sequences", "oid": 12219},
    {"typname": "pg_statio_user_sequences", "oid": 12220},
    {"typname": "_pg_stat_activity", "oid": 12223},
    {"typname": "pg_stat_activity", "oid": 12224},
    {"typname": "_pg_stat_replication", "oid": 12228},
    {"typname": "pg_stat_replication", "oid": 12229},
    {"typname": "_pg_stat_slru", "oid": 12233},
    {"typname": "pg_stat_slru", "oid": 12234},
    {"typname": "_pg_stat_wal_receiver", "oid": 12237},
    {"typname": "pg_stat_wal_receiver", "oid": 12238},
    {"typname": "_pg_stat_recovery_prefetch", "oid": 12241},
    {"typname": "pg_stat_recovery_prefetch", "oid": 12242},
    {"typname": "_pg_stat_subscription", "oid": 12245},
    {"typname": "pg_stat_subscription", "oid": 12246},
    {"typname": "_pg_stat_ssl", "oid": 12250},
    {"typname": "pg_stat_ssl", "oid": 12251},
    {"typname": "_pg_stat_gssapi", "oid": 12254},
    {"typname": "pg_stat_gssapi", "oid": 12255},
    {"typname": "_pg_replication_slots", "oid": 12258},
    {"typname": "pg_replication_slots", "oid": 12259},
    {"typname": "_pg_stat_replication_slots", "oid": 12263},
    {"typname": "pg_stat_replication_slots", "oid": 12264},
    {"typname": "_pg_stat_database", "oid": 12267},
    {"typname": "pg_stat_database", "oid": 12268},
    {"typname": "_pg_stat_database_conflicts", "oid": 12272},
    {"typname": "pg_stat_database_conflicts", "oid": 12273},
    {"typname": "_pg_stat_user_functions", "oid": 12276},
    {"typname": "pg_stat_user_functions", "oid": 12277},
    {"typname": "_pg_stat_xact_user_functions", "oid": 12281},
    {"typname": "pg_stat_xact_user_functions", "oid": 12282},
    {"typname": "_pg_stat_archiver", "oid": 12286},
    {"typname": "pg_stat_archiver", "oid": 12287},
    {"typname": "_pg_stat_bgwriter", "oid": 12290},
    {"typname": "pg_stat_bgwriter", "oid": 12291},
    {"typname": "_pg_stat_wal", "oid": 12294},
    {"typname": "pg_stat_wal", "oid": 12295},
    {"typname": "_pg_stat_progress_analyze", "oid": 12298},
    {"typname": "pg_stat_progress_analyze", "oid": 12299},
    {"typname": "_pg_stat_progress_vacuum", "oid": 12303},
    {"typname": "pg_stat_progress_vacuum", "oid": 12304},
    {"typname": "_pg_stat_progress_cluster", "oid": 12308},
    {"typname": "pg_stat_progress_cluster", "oid": 12309},
    {"typname": "_pg_stat_progress_create_index", "oid": 12313},
    {"typname": "pg_stat_progress_create_index", "oid": 12314},
    {"typname": "_pg_stat_progress_basebackup", "oid": 12318},
    {"typname": "pg_stat_progress_basebackup", "oid": 12319},
    {"typname": "_pg_stat_progress_copy", "oid": 12323},
    {"typname": "pg_stat_progress_copy", "oid": 12324},
    {"typname": "_pg_user_mappings", "oid": 12328},
    {"typname": "pg_user_mappings", "oid": 12329},
    {"typname": "_pg_replication_origin_status", "oid": 12333},
    {"typname": "pg_replication_origin_status", "oid": 12334},
    {"typname": "_pg_stat_subscription_stats", "oid": 12337},
    {"typname": "pg_stat_subscription_stats", "oid": 12338},
    {"typname": "_cardinal_number", "oid": 12416},
    {"typname": "cardinal_number", "oid": 12417},
    {"typname": "_character_data", "oid": 12419},
    {"typname": "character_data", "oid": 12420},
    {"typname": "_sql_identifier", "oid": 12421},
    {"typname": "sql_identifier", "oid": 12422},
    {"typname": "_information_schema_catalog_name", "oid": 12424},
    {"typname": "information_schema_catalog_name", "oid": 12425},
    {"typname": "_time_stamp", "oid": 12427},
    {"typname": "time_stamp", "oid": 12428},
    {"typname": "_yes_or_no", "oid": 12429},
    {"typname": "yes_or_no", "oid": 12430},
    {"typname": "_applicable_roles", "oid": 12433},
    {"typname": "applicable_roles", "oid": 12434},
    {"typname": "_administrable_role_authorizations", "oid": 12438},
    {"typname": "administrable_role_authorizations", "oid": 12439},
    {"typname": "_attributes", "oid": 12442},
    {"typname": "attributes", "oid": 12443},
    {"typname": "_character_sets", "oid": 12447},
    {"typname": "character_sets", "oid": 12448},
    {"typname": "_check_constraint_routine_usage", "oid": 12452},
    {"typname": "check_constraint_routine_usage", "oid": 12453},
    {"typname": "_check_constraints", "oid": 12457},
    {"typname": "check_constraints", "oid": 12458},
    {"typname": "_collations", "oid": 12462},
    {"typname": "collations", "oid": 12463},
    {"typname": "_collation_character_set_applicability", "oid": 12467},
    {"typname": "collation_character_set_applicability", "oid": 12468},
    {"typname": "_column_column_usage", "oid": 12472},
    {"typname": "column_column_usage", "oid": 12473},
    {"typname": "_column_domain_usage", "oid": 12477},
    {"typname": "column_domain_usage", "oid": 12478},
    {"typname": "_column_privileges", "oid": 12482},
    {"typname": "column_privileges", "oid": 12483},
    {"typname": "_column_udt_usage", "oid": 12487},
    {"typname": "column_udt_usage", "oid": 12488},
    {"typname": "_columns", "oid": 12492},
    {"typname": "columns", "oid": 12493},
    {"typname": "_constraint_column_usage", "oid": 12497},
    {"typname": "constraint_column_usage", "oid": 12498},
    {"typname": "_constraint_table_usage", "oid": 12502},
    {"typname": "constraint_table_usage", "oid": 12503},
    {"typname": "_domain_constraints", "oid": 12507},
    {"typname": "domain_constraints", "oid": 12508},
    {"typname": "_domain_udt_usage", "oid": 12512},
    {"typname": "domain_udt_usage", "oid": 12513},
    {"typname": "_domains", "oid": 12517},
    {"typname": "domains", "oid": 12518},
    {"typname": "_enabled_roles", "oid": 12522},
    {"typname": "enabled_roles", "oid": 12523},
    {"typname": "_key_column_usage", "oid": 12526},
    {"typname": "key_column_usage", "oid": 12527},
    {"typname": "_parameters", "oid": 12531},
    {"typname": "parameters", "oid": 12532},
    {"typname": "_referential_constraints", "oid": 12536},
    {"typname": "referential_constraints", "oid": 12537},
    {"typname": "_role_column_grants", "oid": 12541},
    {"typname": "role_column_grants", "oid": 12542},
    {"typname": "_routine_column_usage", "oid": 12545},
    {"typname": "routine_column_usage", "oid": 12546},
    {"typname": "_routine_privileges", "oid": 12550},
    {"typname": "routine_privileges", "oid": 12551},
    {"typname": "_role_routine_grants", "oid": 12555},
    {"typname": "role_routine_grants", "oid": 12556},
    {"typname": "_routine_routine_usage", "oid": 12559},
    {"typname": "routine_routine_usage", "oid": 12560},
    {"typname": "_routine_sequence_usage", "oid": 12564},
    {"typname": "routine_sequence_usage", "oid": 12565},
    {"typname": "_routine_table_usage", "oid": 12569},
    {"typname": "routine_table_usage", "oid": 12570},
    {"typname": "_routines", "oid": 12574},
    {"typname": "routines", "oid": 12575},
    {"typname": "_schemata", "oid": 12579},
    {"typname": "schemata", "oid": 12580},
    {"typname": "_sequences", "oid": 12583},
    {"typname": "sequences", "oid": 12584},
    {"typname": "_sql_features", "oid": 12588},
    {"typname": "sql_features", "oid": 12589},
    {"typname": "_sql_implementation_info", "oid": 12593},
    {"typname": "sql_implementation_info", "oid": 12594},
    {"typname": "_sql_parts", "oid": 12598},
    {"typname": "sql_parts", "oid": 12599},
    {"typname": "_sql_sizing", "oid": 12603},
    {"typname": "sql_sizing", "oid": 12604},
    {"typname": "_table_constraints", "oid": 12608},
    {"typname": "table_constraints", "oid": 12609},
    {"typname": "_table_privileges", "oid": 12613},
    {"typname": "table_privileges", "oid": 12614},
    {"typname": "_role_table_grants", "oid": 12618},
    {"typname": "role_table_grants", "oid": 12619},
    {"typname": "_tables", "oid": 12622},
    {"typname": "tables", "oid": 12623},
    {"typname": "_transforms", "oid": 12627},
    {"typname": "transforms", "oid": 12628},
    {"typname": "_triggered_update_columns", "oid": 12632},
    {"typname": "triggered_update_columns", "oid": 12633},
    {"typname": "_triggers", "oid": 12637},
    {"typname": "triggers", "oid": 12638},
    {"typname": "_udt_privileges", "oid": 12642},
    {"typname": "udt_privileges", "oid": 12643},
    {"typname": "_role_udt_grants", "oid": 12647},
    {"typname": "role_udt_grants", "oid": 12648},
    {"typname": "_usage_privileges", "oid": 12651},
    {"typname": "usage_privileges", "oid": 12652},
    {"typname": "_role_usage_grants", "oid": 12656},
    {"typname": "role_usage_grants", "oid": 12657},
    {"typname": "_user_defined_types", "oid": 12660},
    {"typname": "user_defined_types", "oid": 12661},
    {"typname": "_view_column_usage", "oid": 12665},
    {"typname": "view_column_usage", "oid": 12666},
    {"typname": "_view_routine_usage", "oid": 12670},
    {"typname": "view_routine_usage", "oid": 12671},
    {"typname": "_view_table_usage", "oid": 12675},
    {"typname": "view_table_usage", "oid": 12676},
    {"typname": "_views", "oid": 12680},
    {"typname": "views", "oid": 12681},
    {"typname": "_data_type_privileges", "oid": 12685},
    {"typname": "data_type_privileges", "oid": 12686},
    {"typname": "_element_types", "oid": 12690},
    {"typname": "element_types", "oid": 12691},
    {"typname": "__pg_foreign_table_columns", "oid": 12695},
    {"typname": "_pg_foreign_table_columns", "oid": 12696},
    {"typname": "_column_options", "oid": 12700},
    {"typname": "column_options", "oid": 12701},
    {"typname": "__pg_foreign_data_wrappers", "oid": 12704},
    {"typname": "_pg_foreign_data_wrappers", "oid": 12705},
    {"typname": "_foreign_data_wrapper_options", "oid": 12708},
    {"typname": "foreign_data_wrapper_options", "oid": 12709},
    {"typname": "_foreign_data_wrappers", "oid": 12712},
    {"typname": "foreign_data_wrappers", "oid": 12713},
    {"typname": "__pg_foreign_servers", "oid": 12716},
    {"typname": "_pg_foreign_servers", "oid": 12717},
    {"typname": "_foreign_server_options", "oid": 12721},
    {"typname": "foreign_server_options", "oid": 12722},
    {"typname": "_foreign_servers", "oid": 12725},
    {"typname": "foreign_servers", "oid": 12726},
    {"typname": "__pg_foreign_tables", "oid": 12729},
    {"typname": "_pg_foreign_tables", "oid": 12730},
    {"typname": "_foreign_table_options", "oid": 12734},
    {"typname": "foreign_table_options", "oid": 12735},
    {"typname": "_foreign_tables", "oid": 12738},
    {"typname": "foreign_tables", "oid": 12739},
    {"typname": "__pg_user_mappings", "oid": 12742},
    {"typname": "_pg_user_mappings", "oid": 12743},
    {"typname": "_user_mapping_options", "oid": 12747},
    {"typname": "user_mapping_options", "oid": 12748},
    {"typname": "_user_mappings", "oid": 12752},
    {"typname": "user_mappings", "oid": 12753},
    {"typname": "_crypto_box_keypair", "oid": 16657},
    {"typname": "crypto_box_keypair", "oid": 16658},
    {"typname": "_crypto_sign_keypair", "oid": 16664},
    {"typname": "crypto_sign_keypair", "oid": 16665},
    {"typname": "_crypto_kx_keypair", "oid": 16680},
    {"typname": "crypto_kx_keypair", "oid": 16681},
    {"typname": "_crypto_kx_session", "oid": 16686},
    {"typname": "crypto_kx_session", "oid": 16687},
    {"typname": "_crypto_signcrypt_state_key", "oid": 16754},
    {"typname": "crypto_signcrypt_state_key", "oid": 16755},
    {"typname": "_crypto_signcrypt_keypair", "oid": 16757},
    {"typname": "crypto_signcrypt_keypair", "oid": 16758},
    {"typname": "_key_status", "oid": 16771},
    {"typname": "key_status", "oid": 16772},
    {"typname": "_key_type", "oid": 16781},
    {"typname": "key_type", "oid": 16782},
    {"typname": "_key", "oid": 16789},
    {"typname": "key", "oid": 16790},
    {"typname": "__key_id_context", "oid": 16810},
    {"typname": "_key_id_context", "oid": 16811},
    {"typname": "_valid_key", "oid": 16893},
    {"typname": "valid_key", "oid": 16894},
    {"typname": "_masking_rule", "oid": 16909},
    {"typname": "masking_rule", "oid": 16910},
    {"typname": "_mask_columns", "oid": 16914},
    {"typname": "mask_columns", "oid": 16915},
    {"typname": "_decrypted_key", "oid": 16939},
    {"typname": "decrypted_key", "oid": 16940},
    {"typname": "_pg_stat_statements_info", "oid": 27035},
    {"typname": "pg_stat_statements_info", "oid": 27036},
    {"typname": "_pg_stat_statements", "oid": 27046},
    {"typname": "pg_stat_statements", "oid": 27047},
    {"typname": "_aal_level", "oid": 27105},
    {"typname": "aal_level", "oid": 27106},
    {"typname": "_factor_status", "oid": 27113},
    {"typname": "factor_status", "oid": 27114},
    {"typname": "_factor_type", "oid": 27119},
    {"typname": "factor_type", "oid": 27120},
    {"typname": "_audit_log_entries", "oid": 27142},
    {"typname": "audit_log_entries", "oid": 27143},
    {"typname": "_identities", "oid": 27148},
    {"typname": "identities", "oid": 27149},
    {"typname": "_instances", "oid": 27153},
    {"typname": "instances", "oid": 27154},
    {"typname": "_mfa_amr_claims", "oid": 27158},
    {"typname": "mfa_amr_claims", "oid": 27159},
    {"typname": "_mfa_challenges", "oid": 27163},
    {"typname": "mfa_challenges", "oid": 27164},
    {"typname": "_mfa_factors", "oid": 27168},
    {"typname": "mfa_factors", "oid": 27169},
    {"typname": "_refresh_tokens", "oid": 27173},
    {"typname": "refresh_tokens", "oid": 27174},
    {"typname": "_saml_providers", "oid": 27179},
    {"typname": "saml_providers", "oid": 27180},
    {"typname": "_saml_relay_states", "oid": 27187},
    {"typname": "saml_relay_states", "oid": 27188},
    {"typname": "_schema_migrations", "oid": 27193},
    {"typname": "schema_migrations", "oid": 27194},
    {"typname": "_sessions", "oid": 27196},
    {"typname": "sessions", "oid": 27197},
    {"typname": "_sso_domains", "oid": 27199},
    {"typname": "sso_domains", "oid": 27200},
    {"typname": "_sso_providers", "oid": 27205},
    {"typname": "sso_providers", "oid": 27206},
    {"typname": "_users", "oid": 27215},
    {"typname": "users", "oid": 27216},
    {"typname": "_invoice", "oid": 27228},
    {"typname": "invoice", "oid": 27229},
    {"typname": "_subscription", "oid": 27234},
    {"typname": "subscription", "oid": 27235},
    {"typname": "_transactions", "oid": 27240},
    {"typname": "transactions", "oid": 27241},
    {"typname": "_buckets", "oid": 27247},
    {"typname": "buckets", "oid": 27248},
    {"typname": "_migrations", "oid": 27255},
    {"typname": "migrations", "oid": 27256},
    {"typname": "_objects", "oid": 27259},
    {"typname": "objects", "oid": 27260},
    {"typname": "_code_challenge_method", "oid": 27516},
    {"typname": "code_challenge_method", "oid": 27517},
    {"typname": "_flow_state", "oid": 27522},
    {"typname": "flow_state", "oid": 27523},
    {"typname": "_customers", "oid": 27531},
    {"typname": "customers", "oid": 27532},
]
